﻿/*
 * @file jQuery slideshow plugin for CKEditor
 * Copyright (C) 2012 Alfonso Martínez de Lizarrondo
 * Created for SchoolsUnited
 */

CKEDITOR.plugins.add( 'slideshow',
{
	requires : [ 'dialog' ],

	// translations
	lang : ['en', 'nl'],

	getPlaceholderCss : function()
	{
		return '.cke_slideshow' +
				'{' +
					'padding-bottom:27px;'  +
					'background-color:#a9a9a9;'+
					'background-image: url(' + CKEDITOR.getUrl( this.path + 'placeholder.png' ) + ');' +
					'background-position: center bottom;' +
					'background-repeat: no-repeat;' +
					'border: 1px solid #a9a9a9;' +
					'display:inline-block;' +
				'}';
	},

	onLoad : function()
	{
		// v4
		if (CKEDITOR.addCss)
			CKEDITOR.addCss( this.getPlaceholderCss() );

	},

	init : function( editor )
	{
		var config = editor.config,
			lang = editor.lang.slideshow,
			plugin = this,
			icon = plugin.path + 'slideshow.png';

		CKEDITOR.dialog.add( 'slideshow', plugin.path + 'dialog/slideshow.js');

		editor.addCommand( 'slideshow', new CKEDITOR.dialogCommand( 'slideshow' ) );

		editor.ui.addButton( 'Slideshow',
			{
				label : lang.label,
				command : 'slideshow',
				icon : icon
			} );

		// If the "menu" plugin is loaded, register the menu items.
		if ( editor.addMenuItems )
		{
			editor.addMenuItems(
				{
					slideshow :
					{
						label : lang.edit,
						command : 'slideshow',
						group : 'image',
						icon: icon
					}
				} );
		}

		editor.on( 'doubleclick', function( evt )
			{
				var element = evt.data.element;
				if ( element.is( 'img' ) )
					element = element.getParent();

				if ( element && element.is( 'div' ) && !element.isReadOnly() && element.hasClass( 'cke_slideshow' ) )
				{
					evt.data.dialog = 'slideshow';
				}
			});

		// If the "contextmenu" plugin is loaded, register the listeners.
		if ( editor.contextMenu )
		{
			editor.contextMenu.addListener( function( element, selection )
				{
					if ( element && element.is( 'img' ) )
						element = element.getParent();

					if ( element && element.is( 'div' ) && !element.isReadOnly() && element.hasClass( 'cke_slideshow' ) )
					{
						return { slideshow : CKEDITOR.TRISTATE_OFF };
					}
				});
		}

		// v3
		if (editor.addCss)
			editor.addCss( this.getPlaceholderCss() );

		var dtd	= CKEDITOR.dtd;
		// cheat to allow <link> as child of a <div>
		dtd.div['link']=1;

		// To avoid getting a wrapping '<p>'
		dtd['slideshow']={slideshow:1};
		dtd.$block['slideshow']=1;
		//dtd.$body['slideshow']=1;

	}, // init

	afterInit: function( editor )
	{
		var dataProcessor = editor.dataProcessor,
			htmlFilter = dataProcessor && dataProcessor.htmlFilter,
			dataFilter = dataProcessor && dataProcessor.dataFilter;

		// dataFilter : conversion from html input to internal data
		dataFilter.addRules( {
			elements : {
				'div' : function( element )
				{
					if (!element.attributes["class"])
						return;

					if (element.attributes["class"].indexOf("slideshow_holder") != -1)
					{
						var attributes = element.attributes,
							galleryData = {};

						delete attributes.id;

						galleryData['image'] = attributes['data-image'];
						galleryData['id'] = attributes['data-id'];
						galleryData['width'] = attributes['data-width'];
						galleryData['height'] = attributes['data-height'];
						galleryData['position'] = attributes['data-position'];
						galleryData['titles'] = attributes['data-titles'];
						galleryData['pause'] = attributes['data-pause'];
						galleryData['navigation'] = attributes['data-navigation'];
						galleryData['align'] = attributes['data-align'] + '';

						attributes['class'] = 'cke_slideshow';
						switch (galleryData['align'])
						{
							case 'left':
								attributes['class'] += ' slideshowLeft';
								attributes['style'] = 'float:left; margin-right:25px';
								break;

							case 'right':
								attributes['class'] += ' slideshowRight';
								attributes['style'] = 'float:right; margin-left:25px';
								break;
						}

						attributes.title = editor.lang.slideshow.title;
						attributes['data-cke-realelement'] = encodeURIComponent( CKEDITOR.plugins.slideshow.createHtml(galleryData) );
						attributes['data-cke-real-node-type'] = CKEDITOR.NODE_ELEMENT;
						attributes['data-cke-real-element-type'] = 'slideshow';

						var imgAttributes = {};
						imgAttributes.src = galleryData.image;
						imgAttributes.alt = editor.lang.slideshow.title;
						imgAttributes.style = 'width:' + galleryData['width'] + 'px';

						var div = new CKEDITOR.htmlParser.element('div', attributes);
						var img = new CKEDITOR.htmlParser.element('img', imgAttributes);
						div.add( img );
						return div;
					}
				}
			}
		} );

		// htmlFilter : conversion from internal data to html output.
		htmlFilter.addRules(
			{
				elements :
				{
					div : function( element )
					{
						if ( element.attributes[ 'data-cke-real-element-type' ] == 'slideshow' )
						{
							// remove the div if the img has been removed
							if (element.children.length==0 || element.children[0].name != 'img')
								return false;
						}

						return element;
					}
				}
			});


	} // afterInit
} );

CKEDITOR.plugins.slideshow =
{
	createPlaceholder : function( editor, oldElement, text, isGet )
	{
		var element = (oldElement || new CKEDITOR.dom.element( 'div', editor.document )),
			lang = editor.lang.slideshow;

		element.setAttributes(
			{
				'class' : 'cke_slideshow',
				title : lang.title
			}
		);
		element.data('cke-realelement', encodeURIComponent( text ));
		element.data('cke-real-node-type', CKEDITOR.NODE_ELEMENT);
		element.data('cke-real-element-type', 'slideshow');

		if (!oldElement)
		{
			element.append( 'img' );
			element.getFirst().setAttribute('alt', lang.title);
		}

		if ( isGet )
			return element.getOuterHtml();

		if ( !oldElement )
			editor.insertElement( element );

		return element;
	},
	parse : function( element )
	{
		var data = {};
		data.image = element.data('image');
		data.id = element.data('id');
		data.width = element.data('width');
		data.height = element.data('height');
		data.position = element.data('position');
		data.titles = element.data('titles');
		data.pause = element.data('pause');
		data.navigation = element.data('navigation');
		data.align = element.data('align') || '';

		return data;
	},
	setPlaceholderAttributes : function( element, data )
	{
		element.data('image', data.image);
		element.data('id', data.id);
		element.data('width', data.width);
		element.data('height', data.height);
		element.data('position', data.position);
		element.data('titles', data.titles);
		element.data('pause', data.pause);
		element.data('navigation', data.navigation);
		element.data('align', data.align);

		var theClass = 'cke_slideshow',
			style = '';
		switch (data.align)
		{
			case 'left':
				theClass += ' galleryLeft';
				style = 'float:left; margin-right:25px';
				break;

			case 'right':
				theClass += ' galleryRight';
				style = 'float:right; margin-left:25px';
				break;
		}
		element.setAttribute('class', theClass);
		element.setAttribute('style', style);

		// attributes for the image
		var img = element.getFirst();
		img.setAttribute('src', data.image);
		img.setStyle('width', data.width + 'px');

	},
	createHtml : function( data )
	{
		if (!data.id) {
			var now = new Date();
			data.id = 'slideshow' + now.getFullYear() + now.getMonth() + now.getDate() + now.getHours() + now.getMinutes() + now.getSeconds();
		}
		//if (data.position == 'bottom') //add thumbs height
		//{
		//	data.height = parseInt(data.height) + 70;
		//}
		//else
		//{
		//	data.height = parseInt(data.height);
		//}

		var html = [];

		var theClass = 'slideshow_holder',
			style = '';
		switch (data.align)
		{
			case 'left':
				theClass += ' slideshowLeft';
				style = 'float:left; margin-right:25px;';
				break;

			case 'right':
				theClass += ' slideshowRight';
				style = 'float:right; margin-left:25px;';
				break;
		}
		if (style)
			style = ' style="' + style + ' width:' + data.width + 'px;"';


		html.push('<div class="' + theClass +'" data-id="' + data.id + '" ' + ' data-height="' + data.height +
			'" data-width="' + data.width + '" data-position="' + data.position + '" data-titles="' + data.titles + '" ' +
			' data-pause="' + data.pause + '" data-navigation="' + data.navigation + '" data-image="' + data.image + '" ' +
			' data-align="' + data.align + '" ' + style + '>');
		html.push('<link rel="stylesheet" type="text/css" href="starnet/interface/jquery/jquery.dop.ThumbnailGallery.css" />');
        html.push('<script type="text/javascript\" src=\"starnet/interface/jquery/jquery.js"></script>');
		html.push('<script type="text/javascript" src="starnet/interface/jquery/jquery.mousewheel.js"></script>');
        html.push('<script type="text/javascript" src="starnet/interface/jquery/jquery.jscrollpane.min.js"></script>');
        html.push('<script type="text/javascript" src="starnet/interface/jquery/jquery.dop.ThumbnailGallery.js"></script>');

        html.push('<script type="text/javascript">');
        html.push('    $(document).ready(function(){');
        html.push('        $("#' + data.id + '").DOPThumbnailGallery({');
		html.push('		"ID": 1,'); // what's this id?
		html.push('		"SettingsDataType": "HTML",');
		html.push('		"ContentDataType": "XML",');
		html.push('		"ContentFilePath": "starnet/interface/generate_xml.php?path=' + data.image + '&titles=' + data.titles +'"});');
        html.push('    });');
        html.push('</script>');
		html.push('<div id="' + data.id + '">');
		html.push('<img id="pic' + data.id + '" src="' + data.image + '" style="display:none;">');
		html.push('<ul class="Settings" style="display:none;">');
		html.push('	<li class="Width">' + data.width + '</li>');
		html.push('	<li class="Height">' + data.height + '</li>');
		html.push('	<li class="ThumbnailWidth">60</li>');
        html.push(' <li class="ThumbnailHeight">60</li>');
		html.push('	<li class="ThumbnailBorderSize">2</li>');
		html.push('	<li class="ThumbnailsPosition">' + data.position + '</li>');
		html.push('	<li class="ThumbnailsOverImage">' + data.titles + '</li>'); // is this right?
		html.push('	<li class="SlideshowTime">' + data.pause + '</li>');
		html.push('	<li class="AutoHide">' + data.navigation + '</li>'); // is this right?
		html.push('</ul>');
		html.push('</div>');
		return html.join('\r\n');
	}
};


/*
compatibility between CKEditor 3 and 4
*/
CKEDITOR.addPluginLang = function( plugin, lang, obj )
{
	// v3
	if (CKEDITOR.skins)
	{
		var newObj = {};
		newObj[ plugin ] = obj;
		obj = newObj;
	}
	CKEDITOR.plugins.setLang( plugin, lang, obj );
};
